@@ -218,7 +218,14 @@ int sangoma_tdm_get_hw_coding(int fd, wanpipe_tdm_api_t *tdm_api);
 #endif
 int sangoma_tdm_get_hw_dtmf(int fd, wanpipe_tdm_api_t *tdm_api);
 
+
+#ifndef LIBSANGOMA_TDMAPI_CTRL
+#define LIBSANGOMA_TDMAPI_CTRL 1
+#endif
+int sangoma_open_tdmapi_ctrl(void);
+
 #endif 	/* WANPIPE_TDM_API */
 
-#endif /* _LIBSNAGOMA_H */ 
+
+#endif
 
